import {
  AfterViewInit,
  ChangeDetectionStrategy,
  ChangeDetectorRef,
  Component,
  OnDestroy,
  OnInit,
  ViewChild,
  ViewEncapsulation,
} from '@angular/core';
import {
  FormBuilder,
  FormControl,
  FormGroup,
  Validators,
} from '@angular/forms';
import { MatCheckboxChange } from '@angular/material/checkbox';
import { MatPaginator } from '@angular/material/paginator';
import { MatSort } from '@angular/material/sort';
import {
  debounceTime,
  map,
  merge,
  Observable,
  Subject,
  switchMap,
  takeUntil,
} from 'rxjs';
import { fuseAnimations } from '@fuse/animations';
import { FuseConfirmationService } from '@fuse/services/confirmation';

import { PartnerCalculate } from '../models/partner-calculate';
import { PartnerCalculatePagination } from '../models/partner-calculate-pagination';
import { PartnerCalculateService } from '../services/partner-calculate.service';
import { MatDrawer } from '@angular/material/sidenav';
import { MatTableDataSource } from '@angular/material/table';

@Component({
  selector: 'partner-calculate-list',
  templateUrl: './list.component.html',
  styles: [
    /* language=SCSS */
    `
      .partner-calculate-grid {
        /*                     번호  아이디 등급  입금자 결제 */
        grid-template-columns: 40px 140px 100px auto 140px;

        @screen sm {
          /*                     번호  아이디 등급  입금자 결제   신청 */
          grid-template-columns: 40px 140px 100px auto 140px 140px;
        }

        @screen md {
          /*                     번호  아이디  등급  입금자 결제  신청 */
          grid-template-columns: 40px 140px 100px auto 140px 140px;
        }

        @screen lg {
          /*                     번호  아이디 등급  입금자 결제   신청   */
          grid-template-columns: 40px 100px auto 40px 140px 140px;
        }
      }
    `,
  ],
  encapsulation: ViewEncapsulation.None,
  changeDetection: ChangeDetectionStrategy.OnPush,
  animations: fuseAnimations,
})
export class ListComponent implements OnInit, AfterViewInit, OnDestroy {
  @ViewChild(MatPaginator) private _paginator!: MatPaginator;
  @ViewChild(MatSort) private _sort!: MatSort;

  @ViewChild('drawer') drawer!: MatDrawer;
  drawerMode: 'over' | 'side' = 'side';
  drawerOpened: boolean = true;

  partnerDepositCalculatesDataSource: MatTableDataSource<PartnerCalculate> =
    new MatTableDataSource();

  partnerCalculates$!: Observable<PartnerCalculate[] | undefined>;

  panels: any[] = [];
  selectedPanel: string = 'partnerDepositList';

  __isSearchOpened = false;
  isLoading = false;
  searchInputControl = new FormControl();
  selectedPartnerCalculate?: PartnerCalculate;
  pagination?: PartnerCalculatePagination;

  partnerDepositTableColumns: string[] = [
    'no',
    'username',
    'accountHolder',
    'amount',
    'createdAt',
    'stateChangedAt',
  ];

  private _unsubscribeAll: Subject<any> = new Subject<any>();

  /**
   * Constructor
   */
  constructor(
    private _changeDetectorRef: ChangeDetectorRef,
    private _fuseConfirmationService: FuseConfirmationService,
    private _formBuilder: FormBuilder,
    private _partnerCalculateService: PartnerCalculateService
  ) {}

  // -----------------------------------------------------------------------------------------------------
  // @ Lifecycle hooks
  // -----------------------------------------------------------------------------------------------------

  /**
   * On init
   */
  ngOnInit(): void {
    // Get the pagination
    this._partnerCalculateService.pagination$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((pagination: PartnerCalculatePagination | undefined) => {
        // Update the pagination
        this.pagination = pagination;

        // Mark for check
        this._changeDetectorRef.markForCheck();
      });

    // Get the products
    this._partnerCalculateService.partnerCalculates$
      .pipe(takeUntil(this._unsubscribeAll))
      .subscribe((result: PartnerCalculate[] | undefined) => {
        if (!result) {
          return;
        }
        this.partnerDepositCalculatesDataSource.data = result;
      });
    this.partnerCalculates$ = this._partnerCalculateService.partnerCalculates$;

    this.panels = [
      {
        id: 'partnerDepositList',
        icon: 'account-cash',
        title: '파트너입금 내역',
        description: 'Manage your public profile and private information',
      },
      {
        id: 'partnerWithdrawList',
        icon: 'account-cash',
        title: '파트너출금 내역',
        description: 'Manage your public profile and private information',
      },
    ];
  }

  /**
   * After view init
   */
  ngAfterViewInit(): void {
    if (this._sort && this._paginator) {
      // Set the initial sort
      this._sort.sort({
        id: 'partnerDepositId',
        start: 'asc',
        disableClear: true,
      });

      // Mark for check
      this._changeDetectorRef.markForCheck();

      // If the partnerCalculate changes the sort order...
      this._sort.sortChange
        .pipe(takeUntil(this._unsubscribeAll))
        .subscribe(() => {
          // Reset back to the first page
          this._paginator.pageIndex = 0;
        });

      // Get products if sort or page changes
      merge(this._sort.sortChange, this._paginator.page)
        .pipe(
          switchMap(() => {
            this.isLoading = true;
            return this._partnerCalculateService.getPartnerCalculates(
              this._paginator.pageIndex,
              this._paginator.pageSize,
              this._sort.active,
              this._sort.direction
            );
          }),
          map(() => {
            this.isLoading = false;
          })
        )
        .subscribe();
    }
  }

  /**
   * On destroy
   */
  ngOnDestroy(): void {
    // Unsubscribe from all subscriptions
    this._unsubscribeAll.next(null);
    this._unsubscribeAll.complete();
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Public methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Get the details of the panel
   *
   * @param id
   */
  getPanelInfo(id: string): any {
    return this.panels.find((panel) => panel.id === id);
  }

  /**
   * Navigate to the panel
   *
   * @param panel
   */
  goToPanel(panel: string): void {
    this.selectedPanel = panel;

    // Close the drawer on 'over' mode
    if (this.drawerMode === 'over') {
      this.drawer.close();
    }
  }

  // -----------------------------------------------------------------------------------------------------
  // @ Private methods
  // -----------------------------------------------------------------------------------------------------

  /**
   * Create product
   */
  __createProduct(): void {}

  /**
   * Toggle product details
   *
   * @param productId
   */
  __toggleDetails(productId: string): void {}

  /**
   * toggle the search
   * Used in 'bar'
   */
  __onClickSearch(): void {
    this.__isSearchOpened = !this.__isSearchOpened;
  }

  /**
   * Track by function for ngFor loops
   *
   * @param index
   * @param item
   */
  __trackByFn(index: number, item: any): any {
    return item.id || index;
  }
}
